#include "spawn.h"

#include "..\algorithms.h"

void botInfo::gotHelp(Player *p, Command *c)
{
	// List commands

	if (!*c->final)
	{
		switch (p->access)
		{
		case OP_Duke:
		case OP_Baron:
		case OP_King:
		case OP_Emperor:
		case OP_RockStar:
		case OP_Q:
		case OP_God:
		case OP_Owner:
			{	// Owner-level commands
			}
		case OP_SysOp:
			{	// SysOp-level commands
			}
		case OP_SuperModerator:
			{	// SuperModerator-level commands
			}
		case OP_Moderator:
			{	// Moderator-level commands
			}
		case OP_Limited:
			{	// Limited-level commands
				sendPrivate(p, "!cgive, !ctake !cget !creset");
			}
		case OP_Player:
			{	// Player-level commands
				sendPrivate(p, "!about !creds !buy !donate");
			}
		}

		return;
	}

	// Specific command help

	switch (p->access)
	{
	case OP_Duke:
	case OP_Baron:
	case OP_King:
	case OP_Emperor:
	case OP_RockStar:
	case OP_Q:
	case OP_God:
	case OP_Owner:
		{	// Owner-level commands
			if (c->checkParam("owner") || c->checkParam("all"))
			{
			}
		}
	case OP_SysOp:
		{	// SysOp-level commands
			if (c->checkParam("sop") || c->checkParam("all"))
			{
			}
		}
	case OP_SuperModerator:
		{	// SuperModerator-level commands
			if (c->checkParam("smod") || c->checkParam("all"))
			{
			}
		}
	case OP_Moderator:
		{	// Moderator-level commands
			if (c->checkParam("mod") || c->checkParam("all"))
			{
			}
		}
	case OP_Limited:
		{	// Limited-level commands
			if (c->checkParam("cgive"))
			{
				sendPrivate(p, "!cgive");
				sendPrivate(p, "Syntax: !cgive <player name>:<credits>");
				sendPrivate(p, "Adds credits to a player");
			}
			else if (c->checkParam("ctake"))
			{
				sendPrivate(p, "!ctake");
				sendPrivate(p, "Syntax: !ctake <player name>:<credits>");
				sendPrivate(p, "Removes credits from a player");
			}
			else if (c->checkParam("cget"))
			{
				sendPrivate(p, "!cget");
				sendPrivate(p, "Syntax: !cget <player name>");
				sendPrivate(p, "Shows how many credits a player has");
			}
			else if (c->checkParam("creset"))
			{
				sendPrivate(p, "!creset");
				sendPrivate(p, "Syntax: !creset");
				sendPrivate(p, "Resets all credits");
			}
		}
	case OP_Player:
		{	// Player-level commands
			if (c->checkParam("creds"))
			{
				sendPrivate(p, "!creds");
				sendPrivate(p, "Syntax: !creds");
				sendPrivate(p, "Displays how many credits you have");
			}
			else if (c->checkParam("buy"))
			{
				sendPrivate(p, "!buy");
				sendPrivate(p, "Syntax: !buy [item]");
				sendPrivate(p, "Buys something from the bot");
				sendPrivate(p, "If no item is specified then displays what you can buy");
			}
			else if (c->checkParam("donate"))
			{
				sendPrivate(p, "!donate");
				sendPrivate(p, "Syntax: !donate <player name>:<credits>");
				sendPrivate(p, "Donates some credits to a player");
			}
		}
	}
}

void botInfo::gotCommand(Player *p, Command *c)
{
	if (!p) return;
	if (!c) return;

	switch (p->access)
	{
	case OP_Duke:
	case OP_Baron:
	case OP_King:
	case OP_Emperor:
	case OP_RockStar:
	case OP_Q:
	case OP_God:
	case OP_Owner:
		{	// Owner-level commands
		}
	case OP_SysOp:
		{	// SysOp-level commands
		}
	case OP_SuperModerator:
		{	// SuperModerator-level commands
		}
	case OP_Moderator:
		{	// Moderator-level commands
		}
	case OP_Limited:
		{	// Limited-level commands
			if (c->check("cgive"))
			{
				if (!*c->final)
				{
					sendPrivate(p, "Invalid syntax");
					return;
				}

				String name, creds;
				creds = c->final;
				name = creds.split(':');

				if (name == creds || !isNumeric(creds.msg))
				{
					sendPrivate(p, "Invalid syntax");
					return;
				}

				Sint32 i = getInteger(creds.msg, 10);

				Credit(i, name.msg);

				sendPrivate(p, "Credits added");
			}
			else if (c->check("ctake"))
			{
				if (!*c->final)
				{
					sendPrivate(p, "Invalid syntax");
					return;
				}

				String name, creds;
				creds = c->final;
				name = creds.split(':');

				if (name == creds || !isNumeric(creds.msg))
				{
					sendPrivate(p, "Invalid syntax");
					return;
				}

				Sint32 i = getInteger(creds.msg, 10);

				Credit(-i, name.msg);

				sendPrivate(p, "Credits removed");
			}
			else if (c->check("creset"))
			{
				BankBox.clear();

				sendPrivate(p, "All credits have been cleared");
			}
			else if (c->check("cget"))
			{
				if (!*c->final)
				{
					sendPrivate(p, "Invalid syntax");
					return;
				}

				String s;
				s += c->final;
				s += " has ";
				s += GetCredits(FindBankAccount(c->final));
				s += " credits";
				sendPrivate(p, s.msg);
			}
		}
	case OP_Player:
		{	// Player-level commands
			if (c->check("about"))
			{
				String s;
				s = "\"TM Mer\" 1.14 compiled on ";
				s += __DATE__;
				s += " ";
				s += __TIME__;

				sendPrivate(p, s.msg);

				s = "DLL core version: ";
				s += (int)CORE_MAJOR_VERSION;
				s += ".";
				s += (int)CORE_MINOR_VERSION;

				sendPrivate(p, s.msg);

				sendPrivate(p, "Made by SOS(aarkesos@subdimension.com) in cooperation with Continuum Central Union(ccu.continuum-central.com). Well not really, but they deserve the ad :p");
			}
			else if (c->check("creds"))
			{
				BankAccount *t = FindBankAccount(p->name);

				if (!t)
				{
					sendPrivate(p, "You have never had any credits");
					return;
				}

				String s;
				s = "You have ";
				s += t->Credits;
				s += " credits";

				sendPrivate(p, s.msg);
			}
			else if (c->check("donate"))
			{
				if (!*c->final)
				{
					sendPrivate(p, "Invalid syntax");
					return;
				}

				String name, creds;
				creds = c->final;
				name = creds.split(':');

				if (name == creds || !isNumeric(creds.msg))
				{
					sendPrivate(p, "Invalid syntax");
					return;
				}

				Sint32 i = getInteger(creds.msg, 10);

				if (i < 0)
				{
					sendPrivate(p, "Ill donate you -$5000 first");
					return;
				}

				if (GetCredits(FindBankAccount(p->name)) < i)
				{
					sendPrivate(p, "You do not have enough credits");
					return;
				}

				Credit(i, name.msg);
				Credit(-i, p->name);

				String s = i;
				s += " credits donated to ";
				s += name;
				sendPrivate(p, s.msg);
			}
			else if (c->check("buy"))
			{
				if (!*c->final)
				{
					// Display
					if (!StoreBox.total)
					{
						sendPrivate(p, "There is nothing to buy");
						return;
					}

					sendPrivate(p, "Item name            Price          Description");

					_listnode <StoreItem> *parse = StoreBox.head;
					while (parse)
					{
						String s;
						s = parse->item->Name;
						s += " ";
						s = s.rfill(21);
						s += parse->item->Value;
						s += " ";
						s = s.rfill(36);
						s += parse->item->Description;

						sendPrivate(p, s.msg);

						parse = parse->next;
					}

					sendPrivate(p, "Type !buy <item name> to buy something");
				}
				else
				{
					// Buy
					String name, param;
					param = c->final;
					name = param.split(' ');

					StoreItem *t = FindItem(name.msg);

					if (!t)
					{
						sendPrivate(p, "Unknown item");
						return;
					}

					Sint32 i = t->Value - GetCredits(FindBankAccount(p->name));

					if (i > 0)
					{
						String s;
						s = "You need ";
						s += i;
						s += " more credits to buy that item";

						sendPrivate(p, s.msg);
						return;
					}

					if (t->SpecialPoints && (Sint32)(p->score.flagPoints + p->score.killPoints) < t->SpecialPoints)
					{
						sendPrivate(p, "You do not have enough points");
						return;
					}

					if (t->SpecialPoints)
					{
						String s = "*points ";
						s += t->SpecialPoints;

						sendPrivate(p, s.msg);
					}

					Credit(-t->Value, p->name);

					for (i = 0; i < 10; i++)
						if (t->PrivActions[i].len)
						{
							String s = strReplace(t->PrivActions[i].msg, "%name%", p->name);
							s = strReplace(s.msg, "%points%", getString(p->score.killPoints + p->score.flagPoints, 10, 0, false).msg);
							s = strReplace(s.msg, "%squad%", p->squad);
							s = strReplace(s.msg, "%freq1%", (p->team >= 100 ? "(Private freq)" : getString(p->team, 10, 0, false).msg));
							s = strReplace(s.msg, "%freq2%", getString(p->team, 10, 0, false).msg);
							s = strReplace(s.msg, "%param%", param.msg);
							sendPrivate(p, s.msg);
						}

					for (i = 0; i < 10; i++)
						if (t->PubActions[i].len)
						{
							String s = strReplace(t->PubActions[i].msg, "%name%", p->name);
							s = strReplace(s.msg, "%points%", getString(p->score.killPoints + p->score.flagPoints, 10, 0, false).msg);
							s = strReplace(s.msg, "%squad%", p->squad);
							s = strReplace(s.msg, "%freq1%", (p->team >= 100 ? "(Private freq)" : getString(p->team, 10, 0, false).msg));
							s = strReplace(s.msg, "%freq2%", getString(p->team, 10, 0, false).msg);
							s = strReplace(s.msg, "%param%", param.msg);
							sendPublic(s.msg);
						}

					for (i = 0; i < 10; i++)
						switch (t->SpecialActions[i])
						{
						case 1:
						case 4:
						case 7:
							//1  -> Say carried flags in private
							//4  -> Say carried flags in public
							//7  -> Say carried flags to team
							{
								Uint32 count = 0;

								String msg;
								msg = "Display bought by ";
								msg += p->name;
								if (t->SpecialActions[i] == 4)
									sendPublic(msg.msg);
								else if (t->SpecialActions[i] == 7)
									sendTeamPrivate(p->team, msg.msg);

								_listnode <Player> *parse = playerlist->head;
								while (parse)
								{
									if (parse->item->flagCount)
									{
										count++;
										String s;
										s = parse->item->name;
										s += " @ ";
										s += getCoords(p->pos.x, p->pos.y);
										s += " is carrying ";
										s += parse->item->flagCount;
										s += " flags";
										
										if (t->SpecialActions[i] == 1)
											sendPrivate(p, s.msg);
										else if (t->SpecialActions[i] == 4)
											sendPublic(s.msg);
										else if (t->SpecialActions[i] == 7)
											sendTeamPrivate(p->team, s.msg);
									}

									parse = parse->next;
								}

								if (!count)
								{
									if (t->SpecialActions[i] == 1)
										sendPrivate(p, "There are no carried flags");
									else if (t->SpecialActions[i] == 4)
										sendPublic("There are no carried flags");
									else if (t->SpecialActions[i] == 7)
										sendTeamPrivate(p->team, "There are no carried flags");
								}
							}
							break;
						case 2:
						case 5:
						case 8:
							//2  -> Say uncarried flags in private
							//5  -> Say uncarried flags in public
							//8  -> Say uncarried flags to team
							{
								String msg;
								msg = "Display bought by ";
								msg += p->name;
								if (t->SpecialActions[i] == 5)
									sendPublic(msg.msg);
								else if (t->SpecialActions[i] == 8)
									sendTeamPrivate(p->team, msg.msg);

								if (!flaglist->total)
								{
									if (t->SpecialActions[i] == 2)
										sendPrivate(p, "There are no uncarried flags");
									else if (t->SpecialActions[i] == 5)
										sendPublic("There are no uncarried flags");
									else if (t->SpecialActions[i] == 8)
										sendTeamPrivate(p->team, "There are no uncarried flags");

									break;
								}

								String s;

								_listnode <Flag> *parse = flaglist->head;
								while (parse)
								{
									if (s.len)
										s += " ";

									s += getCoords(parse->item->x << 4, parse->item->y << 4);

									parse = parse->next;
								}

								if (t->SpecialActions[i] == 2)
									sendPrivate(p, s.msg);
								else if (t->SpecialActions[i] == 5)
									sendPublic(s.msg);
								else if (t->SpecialActions[i] == 8)
									sendTeamPrivate(p->team, s.msg);
							}
							break;
						case 3:
						case 6:
						case 9:
							//3  -> Say all flags in private
							//6  -> Say all flags in public
							//9  -> Say all flags to team
							{
								Uint32 count = 0;

								String msg;
								msg = "Display bought by ";
								msg += p->name;
								if (t->SpecialActions[i] == 6)
									sendPublic(msg.msg);
								else if (t->SpecialActions[i] == 9)
									sendTeamPrivate(p->team, msg.msg);

								_listnode <Player> *parse = playerlist->head;
								while (parse)
								{
									if (parse->item->flagCount)
									{
										count++;
										String s;
										s = parse->item->name;
										s += " @ ";
										s += getCoords(p->pos.x, p->pos.y);
										s += " is carrying ";
										s += parse->item->flagCount;
										s += " flags";
										
										if (t->SpecialActions[i] == 3)
											sendPrivate(p, s.msg);
										else if (t->SpecialActions[i] == 6)
											sendPublic(s.msg);
										else if (t->SpecialActions[i] == 9)
											sendTeamPrivate(p->team, s.msg);
									}

									parse = parse->next;
								}

								if (!count)
								{
									if (t->SpecialActions[i] == 3)
										sendPrivate(p, "There are no carried flags");
									else if (t->SpecialActions[i] == 6)
										sendPublic("There are no carried flags");
									else if (t->SpecialActions[i] == 9)
										sendTeamPrivate(p->team, "There are no carried flags");
								}

								String s;

								_listnode <Flag> *fparse = flaglist->head;
								while (fparse)
								{
									if (s.len)
										s += " ";

									s += getCoords(fparse->item->x << 4, fparse->item->y << 4);

									fparse = fparse->next;
								}

								if (!flaglist->total)
								{
									if (t->SpecialActions[i] == 3)
										sendPrivate(p, "There are no uncarried flags");
									else if (t->SpecialActions[i] == 6)
										sendPublic("There are no uncarried flags");
									else if (t->SpecialActions[i] == 9)
										sendTeamPrivate(p->team, "There are no uncarried flags");

									break;
								}

								if (t->SpecialActions[i] == 3)
									sendPrivate(p, s.msg);
								else if (t->SpecialActions[i] == 6)
									sendPublic(s.msg);
								else if (t->SpecialActions[i] == 9)
									sendTeamPrivate(p->team, s.msg);
							}
							break;
						case 10:
						case 13:
						case 16:
							//10 -> Say friendly player locations in private
							//13 -> Say friendly player locations in public
							//16 -> Say friendly player locations to team
							{
								String s;

								String msg;
								msg = "Display bought by ";
								msg += p->name;
								if (t->SpecialActions[i] == 13)
									sendPublic(msg.msg);
								else if (t->SpecialActions[i] == 16)
									sendTeamPrivate(p->team, msg.msg);

								_listnode <Player> *parse = playerlist->head;
								while (parse)
								{
									if (parse->item->team == p->team && parse->item->ship != SHIP_Spectator)
									{
										if (s.len)
											s += "   ";

										s += parse->item->name;
										s += " @ ";
										s += getCoords(parse->item->pos.x, parse->item->pos.y);

										if (s.len > 160)
										{
											if (t->SpecialActions[i] == 10)
												sendPrivate(p, s.msg);
											else if (t->SpecialActions[i] == 13)
												sendPublic(s.msg);
											else if (t->SpecialActions[i] == 16)
												sendTeamPrivate(p->team, s.msg);

											s.clear();
										}
									}

									parse = parse->next;
								}

								if (s.len)
								{
									if (t->SpecialActions[i] == 10)
										sendPrivate(p, s.msg);
									else if (t->SpecialActions[i] == 13)
										sendPublic(s.msg);
									else if (t->SpecialActions[i] == 16)
										sendTeamPrivate(p->team, s.msg);
								}
							}
							break;
						case 11:
						case 14:
						case 17:
							//11 -> Say enemy player locations in private
							//14 -> Say enemy player locations in public
							//17 -> Say enemy player locations to team
							{
								String s;

								String msg;
								msg = "Display bought by ";
								msg += p->name;
								if (t->SpecialActions[i] == 14)
									sendPublic(msg.msg);
								else if (t->SpecialActions[i] == 17)
									sendTeamPrivate(p->team, msg.msg);

								_listnode <Player> *parse = playerlist->head;
								while (parse)
								{
									if (parse->item->team != p->team && parse->item->ship != SHIP_Spectator)
									{
										if (s.len)
											s += "   ";

										s += parse->item->name;
										s += " @ ";
										s += getCoords(parse->item->pos.x, parse->item->pos.y);

										if (s.len > 160)
										{
											if (t->SpecialActions[i] == 11)
												sendPrivate(p, s.msg);
											else if (t->SpecialActions[i] == 14)
												sendPublic(s.msg);
											else if (t->SpecialActions[i] == 17)
												sendTeamPrivate(p->team, s.msg);

											s.clear();
										}
									}

									parse = parse->next;
								}

								if (s.len)
								{
									if (t->SpecialActions[i] == 11)
										sendPrivate(p, s.msg);
									else if (t->SpecialActions[i] == 14)
										sendPublic(s.msg);
									else if (t->SpecialActions[i] == 17)
										sendTeamPrivate(p->team, s.msg);
								}
							}
							break;
						case 12:
						case 15:
						case 18:
							//12 -> Say all player locations in private
							//15 -> Say all player locations in public
							//18 -> Say all player locations to team
							{
								String s;

								String msg;
								msg = "Display bought by ";
								msg += p->name;
								if (t->SpecialActions[i] == 15)
									sendPublic(msg.msg);
								else if (t->SpecialActions[i] == 18)
									sendTeamPrivate(p->team, msg.msg);

								_listnode <Player> *parse = playerlist->head;
								while (parse)
								{
									if (parse->item->ship != SHIP_Spectator)
									{
										if (s.len)
											s += "   ";

										s += parse->item->name;
										s += " @ ";
										s += getCoords(parse->item->pos.x, parse->item->pos.y);

										if (s.len > 160)
										{
											if (t->SpecialActions[i] == 12)
												sendPrivate(p, s.msg);
											else if (t->SpecialActions[i] == 15)
												sendPublic(s.msg);
											else if (t->SpecialActions[i] == 18)
												sendTeamPrivate(p->team, s.msg);

											s.clear();
										}
									}

									parse = parse->next;
								}

								if (s.len)
								{
									if (t->SpecialActions[i] == 12)
										sendPrivate(p, s.msg);
									else if (t->SpecialActions[i] == 15)
										sendPublic(s.msg);
									else if (t->SpecialActions[i] == 18)
										sendTeamPrivate(p->team, s.msg);
								}
							}
							break;
						case 20:
							//20 -> Scorereset team
							{
								sendTeamPrivate(p->team, "*scorereset");
							}
							break;
						case 21:
							//21 -> Scorereset enemies
							{
								_listnode <Player> *parse = playerlist->head;
								while (parse)
								{
									if (parse->item->team != p->team)
										sendPrivate(parse->item, "*scorereset");

									parse = parse->next;
								}
							}
							break;
						case 24:
							//24 -> Shipreset team
							{
								sendTeamPrivate(p->team, "*shipreset");
							}
							break;
						case 25:
							//25 -> Shipreset enemies
							{
								_listnode <Player> *parse = playerlist->head;
								while (parse)
								{
									if (parse->item->team != p->team)
										sendPrivate(parse->item, "*shipreset");

									parse = parse->next;
								}
							}
							break;
						case 0:
							break;
						default:
							{
								sendPrivate(p, "The specified item has invalid data");
							}
							break;
						};
				}
			}
		}
	}
}

void botInfo::gotRemoteHelp(char *p, Command *c, Operator_Level l)
{
	// List commands

	if (!*c->final)
	{
		switch (l)
		{
		case OP_Duke:
		case OP_Baron:
		case OP_King:
		case OP_Emperor:
		case OP_RockStar:
		case OP_Q:
		case OP_God:
		case OP_Owner:
			{	// Owner-level commands
			}
		case OP_SysOp:
			{	// SysOp-level commands
			}
		case OP_SuperModerator:
			{	// SuperModerator-level commands
			}
		case OP_Moderator:
			{	// Moderator-level commands
			}
		case OP_Limited:
			{	// Limited-level commands
			}
		case OP_Player:
			{	// Player-level commands
			}
		}

		return;
	}

	// Specific command help

	switch (l)
	{
	case OP_Duke:
	case OP_Baron:
	case OP_King:
	case OP_Emperor:
	case OP_RockStar:
	case OP_Q:
	case OP_God:
	case OP_Owner:
		{	// Owner-level commands
		}
	case OP_SysOp:
		{	// SysOp-level commands
		}
	case OP_SuperModerator:
		{	// SuperModerator-level commands
		}
	case OP_Moderator:
		{	// Moderator-level commands
		}
	case OP_Limited:
		{	// Limited-level commands
		}
	case OP_Player:
		{	// Player-level commands
		}
	}
}

void botInfo::gotRemote(char *p, Command *c, Operator_Level l)
{
	if (!c) return;

	switch (l)
	{
	case OP_Duke:
	case OP_Baron:
	case OP_King:
	case OP_Emperor:
	case OP_RockStar:
	case OP_Q:
	case OP_God:
	case OP_Owner:
		{	// Owner-level commands
		}
	case OP_SysOp:
		{	// SysOp-level commands
		}
	case OP_SuperModerator:
		{	// SuperModerator-level commands
		}
	case OP_Moderator:
		{	// Moderator-level commands
		}
	case OP_Limited:
		{	// Limited-level commands
		}
	case OP_Player:
		{	// Player-level commands
		}
	}
}
