/*
	MERVBot Spawn-callback by Catid@pacbell.net
*/

#ifndef SPAWN_H
#define SPAWN_H


#include "..\dllcore.h"
#include <windows.h>

union itemInfo
{
	struct
	{
		unsigned shields	: 1;	// Has it or not
		unsigned supers		: 1;
		unsigned burst		: 4;	// Item counts
		unsigned repel		: 4;
		unsigned thor		: 4;
		unsigned brick		: 4;
		unsigned decoy		: 4;
		unsigned rocket		: 4;
		unsigned portal		: 4;
		unsigned pack		: 2;	// Probably used somehow
	};

	Uint32 n;
};

union weaponInfo
{
	struct
	{
		unsigned type		: 5;	// enum Projectile_Types
		unsigned level		: 2;	// Only for bombs/bullets
		unsigned bouncing	: 1;	// Only for bombs
		unsigned emp		: 1;	// Only for bombs
		unsigned isBomb		: 1;	// ? This is a bomb
		unsigned shrapCount	: 5;	// 0-31
		unsigned fireType	: 1;	// Bombs -> Mines, Bullets -> Multifire
	};

	Uint16 n;
};

enum Projectile_Types
{
	// Seen "in the wild"
	PROJ_None,
	PROJ_Bullet,
	PROJ_BBullet,
	PROJ_Bomb,
	PROJ_PBomb,
	PROJ_Repel,
	PROJ_Decoy,
	PROJ_Burst,
	PROJ_Thor,

	// Internal to the bot
	PROJ_InactiveBullet,
	PROJ_Shrapnel
};

enum Weapon_Levels
{
	LVL_One,
	LVL_Two,
	LVL_Three,
	LVL_Four
};

enum Ship_Types
{
	SHIP_Warbird,		// 0
	SHIP_Javelin,
	SHIP_Spider,
	SHIP_Leviathan,
	SHIP_Terrier,
	SHIP_Weasel,
	SHIP_Lancaster,
	SHIP_Shark,
	SHIP_Spectator		// 8
};

enum Prize_Types
{
	PRIZE_Unknown,		// 0
	PRIZE_Recharge,
	PRIZE_Energy,
	PRIZE_Rotation,
	PRIZE_Stealth,
	PRIZE_Cloak,
	PRIZE_XRadar,
	PRIZE_Warp,
	PRIZE_Guns,
	PRIZE_Bombs,
	PRIZE_BBullets,
	PRIZE_Thruster,
	PRIZE_TopSpeed,
	PRIZE_FullCharge,
	PRIZE_EngineShutdown,
	PRIZE_Multifire,
	PRIZE_Proximity,
	PRIZE_Super,
	PRIZE_Shields,
	PRIZE_Antiwarp,
	PRIZE_Repel,
	PRIZE_Burst,
	PRIZE_Decoy,
	PRIZE_Thor,
	PRIZE_Multiprize,
	PRIZE_Brick,
	PRIZE_Rocket,
	PRIZE_Portal,		// 28
};

enum Chat_Modes
{
    MSG_Arena,			// 00 Same for *arena, *zone, *szone, **
    MSG_PublicMacro,	// 01 Macro calcs are done client-side
    MSG_Public,			// 02 Public chat
    MSG_Team,			// 03 (//) or (')
    MSG_TeamPrivate,	// 04 Player to all members of another team (")
    MSG_Private,		// 05 Only shows messages addressed to the bot
    MSG_PlayerWarning,	// 06 Red message, with a name tag
    MSG_RemotePrivate,	// 07 Do not trust the sender with any private information.
    MSG_ServerError,	// 08 Red server errors, without a name tag
    MSG_Channel			// 09 Arrives in the same format SubSpace displays
};

struct BankAccount
{
	char Owner[20];
	Sint32 Credits;

	BankAccount()
	{
		ZeroMemory(Owner, 20);
		Credits = 0;
	}
};

struct StoreItem
{
	String Name;
	String Description;
	Sint32 Value;

	String PrivActions[10];
	String PubActions[10];
	Uint32 SpecialActions[10];

	Sint32 SpecialPoints;
};

class botInfo
{
	bool CONNECTION_DENIED;

	CALL_HANDLE handle;
	CALL_COMMAND callback;
	CALL_PLIST playerlist;
	CALL_FLIST flaglist;
	char *arena;
	arenaSettings *settings;
	Player *me;
	BYTE *map;

	int countdown[4];

	/************************/ 

	_linkedlist <StoreItem> StoreBox;
	_linkedlist <BankAccount> BankBox;

	/************************/ 

	bool AllowNegativeCredits;

	Sint32 creSoccerGoalEnemy;
	Sint32 creSoccerGoalTeam;
	Sint32 creFlagVictoryTeam;
	Sint32 creFlagVictoryEnemy;
	Sint32 creKill;
	Sint32 creTeamKill;
	Sint32 creKilled;
	Sint32 creTeamKilled;

	Sint32 crePrize[57];	// +28

	/************************/ 

public:
	botInfo(CALL_HANDLE given)
	{
		handle = given;
		callback = 0;
		playerlist = 0;
		countdown[0] = 300;
		countdown[1] = 0;
		countdown[2] = 0;
		countdown[3] = 0;
		CONNECTION_DENIED = false;
		arena = "<Unknown>";

		AllowNegativeCredits = false;
	}

	/************************/ 

	StoreItem *FindItem(char *Name);
	void LoadStoreFromDisk();
	void LoadCredits();

	/************************/ 

	BankAccount *FindBankAccount(char *Owner);
	void Credit(Sint32 Credits, char *Owner);

	void LoadBankFromDisk();
	void SaveBankToDisk();

	inline Sint32 GetCredits(BankAccount *a)
	{
		if (a)
			return a->Credits;
		else
			return 0;
	}

	/************************/ 

	bool boolVal(char *msg);
	Sint32 coordNumeric(Sint32 y);
	char coordAlpha(Sint32 x);
	String getCoords(Sint32 x, Sint32 y);
	Uint32 getNonSpec();
	Uint32 CountTeams();

	Uint32 strFirstInstanceOf(char *msg, char *msg2);
	String strReplace(char *msg, char *from, char *to);

	/************************/ 

	void gotEvent(BotEvent &event);

	void tell(BotEvent &event);

	bool validate(CALL_HANDLE given) { return given == handle; }

	void sendPrivate(Player *player, char *msg);
	void sendPrivate(Player *player, BYTE snd, char *msg);

	void sendTeam(char *msg);
	void sendTeam(BYTE snd, char *msg);

	void sendTeamPrivate(Uint16 team, char *msg);
	void sendTeamPrivate(Uint16 team, BYTE snd, char *msg);

	void sendPublic(char *msg);
	void sendPublic(BYTE snd, char *msg);

	void sendPublicMacro(char *msg);
	void sendPublicMacro(BYTE snd, char *msg);

	void sendChannel(char *msg);			// #;Message
	void sendRemotePrivate(char *msg);		// :Name:Messsage
	void sendRemotePrivate(char *name, char *msg);

	void gotHelp(Player *p, Command *c);
	void gotCommand(Player *p, Command *c);
	void gotRemoteHelp(char *p, Command *c, Operator_Level l);
	void gotRemote(char *p, Command *c, Operator_Level l);
};


// The botlist contains every bot to ever be spawned of this type,
// some entries may no longer exist.

extern _linkedlist <botInfo> botlist;

botInfo *findBot(CALL_HANDLE handle);


#endif	// SPAWN_H
